# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import compute_pb2 as v1_dot_rules_dot_compute__pb2


class ComputeServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ComputeCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeCreate",
            request_serializer=v1_dot_rules_dot_compute__pb2.ComputeCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_compute__pb2.ComputeCreateResponse.FromString,
        )
        self.ComputeRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeRead",
            request_serializer=v1_dot_rules_dot_compute__pb2.ComputeReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_compute__pb2.ComputeReadResponse.FromString,
        )
        self.ComputeUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeUpdate",
            request_serializer=v1_dot_rules_dot_compute__pb2.ComputeUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_compute__pb2.ComputeUpdateResponse.FromString,
        )
        self.ComputeDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeDelete",
            request_serializer=v1_dot_rules_dot_compute__pb2.ComputeDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_compute__pb2.ComputeDeleteResponse.FromString,
        )


class ComputeServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ComputeCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ComputeRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ComputeUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ComputeDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ComputeServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ComputeCreate": grpc.unary_unary_rpc_method_handler(
            servicer.ComputeCreate,
            request_deserializer=v1_dot_rules_dot_compute__pb2.ComputeCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_compute__pb2.ComputeCreateResponse.SerializeToString,
        ),
        "ComputeRead": grpc.unary_unary_rpc_method_handler(
            servicer.ComputeRead,
            request_deserializer=v1_dot_rules_dot_compute__pb2.ComputeReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_compute__pb2.ComputeReadResponse.SerializeToString,
        ),
        "ComputeUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.ComputeUpdate,
            request_deserializer=v1_dot_rules_dot_compute__pb2.ComputeUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_compute__pb2.ComputeUpdateResponse.SerializeToString,
        ),
        "ComputeDelete": grpc.unary_unary_rpc_method_handler(
            servicer.ComputeDelete,
            request_deserializer=v1_dot_rules_dot_compute__pb2.ComputeDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_compute__pb2.ComputeDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.compute.ComputeService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ComputeService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ComputeCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeCreate",
            v1_dot_rules_dot_compute__pb2.ComputeCreateRequest.SerializeToString,
            v1_dot_rules_dot_compute__pb2.ComputeCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ComputeRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeRead",
            v1_dot_rules_dot_compute__pb2.ComputeReadRequest.SerializeToString,
            v1_dot_rules_dot_compute__pb2.ComputeReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ComputeUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeUpdate",
            v1_dot_rules_dot_compute__pb2.ComputeUpdateRequest.SerializeToString,
            v1_dot_rules_dot_compute__pb2.ComputeUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ComputeDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.compute.ComputeService/ComputeDelete",
            v1_dot_rules_dot_compute__pb2.ComputeDeleteRequest.SerializeToString,
            v1_dot_rules_dot_compute__pb2.ComputeDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/sales/client.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x15v1/sales/client.proto\x12 pro.omni.oms.api.v1.sales.client\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto"\xbb\x01\n\x06\x43lient\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x15\n\rclient_doc_id\x18\x03 \x01(\t\x12*\n\x06\x61\x63tive\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x05 \x01(\t\x12?\n\x0cobject_audit\x18\x06 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\x87\x01\n\x13\x43lientCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rclient_doc_id\x18\x02 \x01(\t\x12\x13\n\x0b\x65xternal_id\x18\x03 \x01(\t\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9b\x01\n\x14\x43lientCreateResponse\x12\x38\n\x06\x63lient\x18\x01 \x01(\x0b\x32(.pro.omni.oms.api.v1.sales.client.Client\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xef\x02\n\x11\x43lientReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd5\x01\n\x12\x43lientReadResponse\x12\x39\n\x07\x63lients\x18\x01 \x03(\x0b\x32(.pro.omni.oms.api.v1.sales.client.Client\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x03 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData"\x87\x01\n\x13\x43lientUpdateRequest\x12\x38\n\x06\x63lient\x18\x01 \x01(\x0b\x32(.pro.omni.oms.api.v1.sales.client.Client\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9b\x01\n\x14\x43lientUpdateResponse\x12\x38\n\x06\x63lient\x18\x01 \x01(\x0b\x32(.pro.omni.oms.api.v1.sales.client.Client\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"Y\n\x13\x43lientDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"a\n\x14\x43lientDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\x8d\x04\n\rClientService\x12\x7f\n\x0c\x43lientCreate\x12\x35.pro.omni.oms.api.v1.sales.client.ClientCreateRequest\x1a\x36.pro.omni.oms.api.v1.sales.client.ClientCreateResponse"\x00\x12y\n\nClientRead\x12\x33.pro.omni.oms.api.v1.sales.client.ClientReadRequest\x1a\x34.pro.omni.oms.api.v1.sales.client.ClientReadResponse"\x00\x12\x7f\n\x0c\x43lientUpdate\x12\x35.pro.omni.oms.api.v1.sales.client.ClientUpdateRequest\x1a\x36.pro.omni.oms.api.v1.sales.client.ClientUpdateResponse"\x00\x12\x7f\n\x0c\x43lientDelete\x12\x35.pro.omni.oms.api.v1.sales.client.ClientDeleteRequest\x1a\x36.pro.omni.oms.api.v1.sales.client.ClientDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.sales.client_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_CLIENT"]._serialized_start = 111
    _globals["_CLIENT"]._serialized_end = 298
    _globals["_CLIENTCREATEREQUEST"]._serialized_start = 301
    _globals["_CLIENTCREATEREQUEST"]._serialized_end = 436
    _globals["_CLIENTCREATERESPONSE"]._serialized_start = 439
    _globals["_CLIENTCREATERESPONSE"]._serialized_end = 594
    _globals["_CLIENTREADREQUEST"]._serialized_start = 597
    _globals["_CLIENTREADREQUEST"]._serialized_end = 964
    _globals["_CLIENTREADRESPONSE"]._serialized_start = 967
    _globals["_CLIENTREADRESPONSE"]._serialized_end = 1180
    _globals["_CLIENTUPDATEREQUEST"]._serialized_start = 1183
    _globals["_CLIENTUPDATEREQUEST"]._serialized_end = 1318
    _globals["_CLIENTUPDATERESPONSE"]._serialized_start = 1321
    _globals["_CLIENTUPDATERESPONSE"]._serialized_end = 1476
    _globals["_CLIENTDELETEREQUEST"]._serialized_start = 1478
    _globals["_CLIENTDELETEREQUEST"]._serialized_end = 1567
    _globals["_CLIENTDELETERESPONSE"]._serialized_start = 1569
    _globals["_CLIENTDELETERESPONSE"]._serialized_end = 1666
    _globals["_CLIENTSERVICE"]._serialized_start = 1669
    _globals["_CLIENTSERVICE"]._serialized_end = 2194
# @@protoc_insertion_point(module_scope)



from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from omni.pro.protos.common import base_pb2 as _base_pb2
from omni.pro.protos.v1.rules import delivery_category_pb2 as _delivery_category_pb2
from omni.pro.protos.v1.rules import delivery_locality_pb2 as _delivery_locality_pb2
from omni.pro.protos.v1.rules import delivery_method_pb2 as _delivery_method_pb2
from omni.pro.protos.v1.rules import warehouse_pb2 as _warehouse_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class DeliveryTime(_message.Message):
    __slots__ = [
        "id",
        "name",
        "code",
        "delivery_methods",
        "warehouses_to",
        "warehouses_from",
        "locality_available",
        "time_type",
        "value_min",
        "value_max",
        "inversely",
        "variable_factor",
        "operator_factor",
        "amount_factor",
        "category_template_not_allowed",
        "active",
        "external_id",
        "object_audit",
    ]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    DELIVERY_METHODS_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSES_TO_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSES_FROM_FIELD_NUMBER: _ClassVar[int]
    LOCALITY_AVAILABLE_FIELD_NUMBER: _ClassVar[int]
    TIME_TYPE_FIELD_NUMBER: _ClassVar[int]
    VALUE_MIN_FIELD_NUMBER: _ClassVar[int]
    VALUE_MAX_FIELD_NUMBER: _ClassVar[int]
    INVERSELY_FIELD_NUMBER: _ClassVar[int]
    VARIABLE_FACTOR_FIELD_NUMBER: _ClassVar[int]
    OPERATOR_FACTOR_FIELD_NUMBER: _ClassVar[int]
    AMOUNT_FACTOR_FIELD_NUMBER: _ClassVar[int]
    CATEGORY_TEMPLATE_NOT_ALLOWED_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    OBJECT_AUDIT_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    code: str
    delivery_methods: _containers.RepeatedCompositeFieldContainer[_delivery_method_pb2.DeliveryMethod]
    warehouses_to: _containers.RepeatedCompositeFieldContainer[_warehouse_pb2.Warehouse]
    warehouses_from: _containers.RepeatedCompositeFieldContainer[_warehouse_pb2.Warehouse]
    locality_available: _delivery_locality_pb2.DeliveryLocality
    time_type: str
    value_min: int
    value_max: int
    inversely: _wrappers_pb2.BoolValue
    variable_factor: str
    operator_factor: str
    amount_factor: float
    category_template_not_allowed: _delivery_category_pb2.DeliveryCategory
    active: _wrappers_pb2.BoolValue
    external_id: str
    object_audit: _base_pb2.ObjectAudit
    def __init__(
        self,
        id: _Optional[str] = ...,
        name: _Optional[str] = ...,
        code: _Optional[str] = ...,
        delivery_methods: _Optional[_Iterable[_Union[_delivery_method_pb2.DeliveryMethod, _Mapping]]] = ...,
        warehouses_to: _Optional[_Iterable[_Union[_warehouse_pb2.Warehouse, _Mapping]]] = ...,
        warehouses_from: _Optional[_Iterable[_Union[_warehouse_pb2.Warehouse, _Mapping]]] = ...,
        locality_available: _Optional[_Union[_delivery_locality_pb2.DeliveryLocality, _Mapping]] = ...,
        time_type: _Optional[str] = ...,
        value_min: _Optional[int] = ...,
        value_max: _Optional[int] = ...,
        inversely: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        variable_factor: _Optional[str] = ...,
        operator_factor: _Optional[str] = ...,
        amount_factor: _Optional[float] = ...,
        category_template_not_allowed: _Optional[_Union[_delivery_category_pb2.DeliveryCategory, _Mapping]] = ...,
        active: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        external_id: _Optional[str] = ...,
        object_audit: _Optional[_Union[_base_pb2.ObjectAudit, _Mapping]] = ...,
    ) -> None: ...

class DeliveryTimeCreateRequest(_message.Message):
    __slots__ = [
        "name",
        "code",
        "locality_available_id",
        "time_type",
        "value_min",
        "value_max",
        "warehouse_to_ids",
        "warehouse_from_ids",
        "delivery_method_ids",
        "inversely",
        "external_id",
        "variable_factor",
        "operator_factor",
        "amount_factor",
        "category_template_not_allowed_id",
        "context",
    ]
    NAME_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    LOCALITY_AVAILABLE_ID_FIELD_NUMBER: _ClassVar[int]
    TIME_TYPE_FIELD_NUMBER: _ClassVar[int]
    VALUE_MIN_FIELD_NUMBER: _ClassVar[int]
    VALUE_MAX_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_TO_IDS_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FROM_IDS_FIELD_NUMBER: _ClassVar[int]
    DELIVERY_METHOD_IDS_FIELD_NUMBER: _ClassVar[int]
    INVERSELY_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    VARIABLE_FACTOR_FIELD_NUMBER: _ClassVar[int]
    OPERATOR_FACTOR_FIELD_NUMBER: _ClassVar[int]
    AMOUNT_FACTOR_FIELD_NUMBER: _ClassVar[int]
    CATEGORY_TEMPLATE_NOT_ALLOWED_ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    name: str
    code: str
    locality_available_id: str
    time_type: str
    value_min: int
    value_max: int
    warehouse_to_ids: _containers.RepeatedScalarFieldContainer[int]
    warehouse_from_ids: _containers.RepeatedScalarFieldContainer[int]
    delivery_method_ids: _containers.RepeatedScalarFieldContainer[str]
    inversely: _wrappers_pb2.BoolValue
    external_id: str
    variable_factor: str
    operator_factor: str
    amount_factor: float
    category_template_not_allowed_id: str
    context: _base_pb2.Context
    def __init__(
        self,
        name: _Optional[str] = ...,
        code: _Optional[str] = ...,
        locality_available_id: _Optional[str] = ...,
        time_type: _Optional[str] = ...,
        value_min: _Optional[int] = ...,
        value_max: _Optional[int] = ...,
        warehouse_to_ids: _Optional[_Iterable[int]] = ...,
        warehouse_from_ids: _Optional[_Iterable[int]] = ...,
        delivery_method_ids: _Optional[_Iterable[str]] = ...,
        inversely: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        external_id: _Optional[str] = ...,
        variable_factor: _Optional[str] = ...,
        operator_factor: _Optional[str] = ...,
        amount_factor: _Optional[float] = ...,
        category_template_not_allowed_id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class DeliveryTimeCreateResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class DeliveryTimeReadRequest(_message.Message):
    __slots__ = ["group_by", "sort_by", "fields", "filter", "paginated", "id", "context"]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    SORT_BY_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    PAGINATED_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    group_by: _containers.RepeatedCompositeFieldContainer[_base_pb2.GroupBy]
    sort_by: _base_pb2.SortBy
    fields: _base_pb2.Fields
    filter: _base_pb2.Filter
    paginated: _base_pb2.Paginated
    id: str
    context: _base_pb2.Context
    def __init__(
        self,
        group_by: _Optional[_Iterable[_Union[_base_pb2.GroupBy, _Mapping]]] = ...,
        sort_by: _Optional[_Union[_base_pb2.SortBy, _Mapping]] = ...,
        fields: _Optional[_Union[_base_pb2.Fields, _Mapping]] = ...,
        filter: _Optional[_Union[_base_pb2.Filter, _Mapping]] = ...,
        paginated: _Optional[_Union[_base_pb2.Paginated, _Mapping]] = ...,
        id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class DeliveryTimeReadResponse(_message.Message):
    __slots__ = ["delivery_times", "meta_data", "response_standard"]
    DELIVERY_TIMES_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_times: _containers.RepeatedCompositeFieldContainer[DeliveryTime]
    meta_data: _base_pb2.MetaData
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_times: _Optional[_Iterable[_Union[DeliveryTime, _Mapping]]] = ...,
        meta_data: _Optional[_Union[_base_pb2.MetaData, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class DeliveryTimeUpdateRequest(_message.Message):
    __slots__ = ["delivery_time", "context"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class DeliveryTimeUpdateResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class DeliveryTimeDeleteRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: str
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[str] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class DeliveryTimeDeleteResponse(_message.Message):
    __slots__ = ["response_standard"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    def __init__(self, response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...) -> None: ...

class AddWarehousesToRequest(_message.Message):
    __slots__ = ["delivery_time_id", "warehouse_to_ids", "context"]
    DELIVERY_TIME_ID_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_TO_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_time_id: str
    warehouse_to_ids: _containers.RepeatedScalarFieldContainer[int]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_time_id: _Optional[str] = ...,
        warehouse_to_ids: _Optional[_Iterable[int]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AddWarehousesToResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class RemoveWarehousesToRequest(_message.Message):
    __slots__ = ["delivery_time_id", "warehouse_to_ids", "context"]
    DELIVERY_TIME_ID_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_TO_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_time_id: str
    warehouse_to_ids: _containers.RepeatedScalarFieldContainer[int]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_time_id: _Optional[str] = ...,
        warehouse_to_ids: _Optional[_Iterable[int]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class RemoveWarehousesToResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AddWarehousesFromRequest(_message.Message):
    __slots__ = ["delivery_time_id", "warehouse_from_ids", "context"]
    DELIVERY_TIME_ID_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FROM_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_time_id: str
    warehouse_from_ids: _containers.RepeatedScalarFieldContainer[int]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_time_id: _Optional[str] = ...,
        warehouse_from_ids: _Optional[_Iterable[int]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AddWarehousesFromResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class RemoveWarehousesFromRequest(_message.Message):
    __slots__ = ["delivery_time_id", "warehouse_from_ids", "context"]
    DELIVERY_TIME_ID_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FROM_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_time_id: str
    warehouse_from_ids: _containers.RepeatedScalarFieldContainer[int]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_time_id: _Optional[str] = ...,
        warehouse_from_ids: _Optional[_Iterable[int]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class RemoveWarehousesFromResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AddDeliveryMethodRequest(_message.Message):
    __slots__ = ["delivery_time_id", "delivery_method_ids", "context"]
    DELIVERY_TIME_ID_FIELD_NUMBER: _ClassVar[int]
    DELIVERY_METHOD_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_time_id: str
    delivery_method_ids: _containers.RepeatedScalarFieldContainer[str]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_time_id: _Optional[str] = ...,
        delivery_method_ids: _Optional[_Iterable[str]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AddDeliveryMethodResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class RemoveDeliveryMethodRequest(_message.Message):
    __slots__ = ["delivery_time_id", "delivery_method_ids", "context"]
    DELIVERY_TIME_ID_FIELD_NUMBER: _ClassVar[int]
    DELIVERY_METHOD_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_time_id: str
    delivery_method_ids: _containers.RepeatedScalarFieldContainer[str]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_time_id: _Optional[str] = ...,
        delivery_method_ids: _Optional[_Iterable[str]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class RemoveDeliveryMethodResponse(_message.Message):
    __slots__ = ["delivery_time", "response_standard"]
    DELIVERY_TIME_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_time: DeliveryTime
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_time: _Optional[_Union[DeliveryTime, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...



from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from omni.pro.protos.common import base_pb2 as _base_pb2
from omni.pro.protos.v1.utilities import model_pb2 as _model_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class ImportExport(_message.Message):
    __slots__ = [
        "id",
        "micro",
        "type_operation",
        "model",
        "name_file",
        "type_file",
        "date_load",
        "date_process",
        "status",
        "message",
        "skip_error",
        "active",
        "object_audit",
    ]
    ID_FIELD_NUMBER: _ClassVar[int]
    MICRO_FIELD_NUMBER: _ClassVar[int]
    TYPE_OPERATION_FIELD_NUMBER: _ClassVar[int]
    MODEL_FIELD_NUMBER: _ClassVar[int]
    NAME_FILE_FIELD_NUMBER: _ClassVar[int]
    TYPE_FILE_FIELD_NUMBER: _ClassVar[int]
    DATE_LOAD_FIELD_NUMBER: _ClassVar[int]
    DATE_PROCESS_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    SKIP_ERROR_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    OBJECT_AUDIT_FIELD_NUMBER: _ClassVar[int]
    id: str
    micro: str
    type_operation: str
    model: _model_pb2.Model
    name_file: str
    type_file: str
    date_load: _timestamp_pb2.Timestamp
    date_process: _timestamp_pb2.Timestamp
    status: str
    message: str
    skip_error: _wrappers_pb2.BoolValue
    active: _wrappers_pb2.BoolValue
    object_audit: _base_pb2.ObjectAudit
    def __init__(
        self,
        id: _Optional[str] = ...,
        micro: _Optional[str] = ...,
        type_operation: _Optional[str] = ...,
        model: _Optional[_Union[_model_pb2.Model, _Mapping]] = ...,
        name_file: _Optional[str] = ...,
        type_file: _Optional[str] = ...,
        date_load: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        date_process: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        status: _Optional[str] = ...,
        message: _Optional[str] = ...,
        skip_error: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        active: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        object_audit: _Optional[_Union[_base_pb2.ObjectAudit, _Mapping]] = ...,
    ) -> None: ...

class ImportExportCreateRequest(_message.Message):
    __slots__ = [
        "micro",
        "type_operation",
        "model_id",
        "name_file",
        "type_file",
        "date_load",
        "date_process",
        "status",
        "message",
        "skip_error",
        "context",
    ]
    MICRO_FIELD_NUMBER: _ClassVar[int]
    TYPE_OPERATION_FIELD_NUMBER: _ClassVar[int]
    MODEL_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FILE_FIELD_NUMBER: _ClassVar[int]
    TYPE_FILE_FIELD_NUMBER: _ClassVar[int]
    DATE_LOAD_FIELD_NUMBER: _ClassVar[int]
    DATE_PROCESS_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    SKIP_ERROR_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    micro: str
    type_operation: str
    model_id: str
    name_file: str
    type_file: str
    date_load: _timestamp_pb2.Timestamp
    date_process: _timestamp_pb2.Timestamp
    status: str
    message: str
    skip_error: _wrappers_pb2.BoolValue
    context: _base_pb2.Context
    def __init__(
        self,
        micro: _Optional[str] = ...,
        type_operation: _Optional[str] = ...,
        model_id: _Optional[str] = ...,
        name_file: _Optional[str] = ...,
        type_file: _Optional[str] = ...,
        date_load: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        date_process: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        status: _Optional[str] = ...,
        message: _Optional[str] = ...,
        skip_error: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class ImportExportCreateResponse(_message.Message):
    __slots__ = ["response_standard", "import_export"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    IMPORT_EXPORT_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    import_export: ImportExport
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        import_export: _Optional[_Union[ImportExport, _Mapping]] = ...,
    ) -> None: ...

class ImportExportReadRequest(_message.Message):
    __slots__ = ["group_by", "sort_by", "fields", "filter", "paginated", "id", "context"]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    SORT_BY_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    PAGINATED_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    group_by: _containers.RepeatedCompositeFieldContainer[_base_pb2.GroupBy]
    sort_by: _base_pb2.SortBy
    fields: _base_pb2.Fields
    filter: _base_pb2.Filter
    paginated: _base_pb2.Paginated
    id: str
    context: _base_pb2.Context
    def __init__(
        self,
        group_by: _Optional[_Iterable[_Union[_base_pb2.GroupBy, _Mapping]]] = ...,
        sort_by: _Optional[_Union[_base_pb2.SortBy, _Mapping]] = ...,
        fields: _Optional[_Union[_base_pb2.Fields, _Mapping]] = ...,
        filter: _Optional[_Union[_base_pb2.Filter, _Mapping]] = ...,
        paginated: _Optional[_Union[_base_pb2.Paginated, _Mapping]] = ...,
        id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class ImportExportReadResponse(_message.Message):
    __slots__ = ["response_standard", "meta_data", "import_export"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    IMPORT_EXPORT_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    meta_data: _base_pb2.MetaData
    import_export: _containers.RepeatedCompositeFieldContainer[ImportExport]
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        meta_data: _Optional[_Union[_base_pb2.MetaData, _Mapping]] = ...,
        import_export: _Optional[_Iterable[_Union[ImportExport, _Mapping]]] = ...,
    ) -> None: ...

class ImportExportUpdateRequest(_message.Message):
    __slots__ = ["import_export", "context"]
    IMPORT_EXPORT_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    import_export: ImportExport
    context: _base_pb2.Context
    def __init__(
        self,
        import_export: _Optional[_Union[ImportExport, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class ImportExportUpdateResponse(_message.Message):
    __slots__ = ["response_standard", "import_export"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    IMPORT_EXPORT_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    import_export: ImportExport
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        import_export: _Optional[_Union[ImportExport, _Mapping]] = ...,
    ) -> None: ...

class ImportExportDeleteRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: str
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[str] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class ImportExportDeleteResponse(_message.Message):
    __slots__ = ["response_standard", "import_export"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    IMPORT_EXPORT_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    import_export: ImportExport
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        import_export: _Optional[_Union[ImportExport, _Mapping]] = ...,
    ) -> None: ...



import ast
import json
import operator
import time

import fakeredis
import mongoengine as mongo
import redis
from bson import ObjectId
from omni.pro.config import Config
from omni.pro.logger import configure_logger
from omni.pro.protos.common import base_pb2
from omni.pro.util import nested
from sqlalchemy import and_, asc, create_engine, desc, not_, or_
from sqlalchemy.orm import aliased, scoped_session, sessionmaker

logger = configure_logger(name=__name__)


class DatabaseManager(object):
    def __init__(self, host: str, port: int, db: str, user: str, password: str, complement: dict) -> None:
        """
        :param db_object: Database object
        Example:
            db_object = {
                "host":"mongo",
                "port":"27017",
                "user":"root",
                "password":"123456",
                "type":"write | read",
                "no_sql":"true",
                "complement":""
            }
        """
        self.db = db
        self.host = host
        self.port = port
        self.username = user
        self.password = password
        self.complement = complement
        # self.get_connection().connect()

    def get_connection(self):
        return MongoConnection(
            db=self.db,
            host=self.host,
            port=self.port,
            username=self.username,
            password=self.password,
            complement=self.complement,
        )

    def create_document(self, db_name: str, document_class, **kwargs) -> object:
        document = document_class(**kwargs)
        document.save()
        return document

    def get_document(self, db_name: str, tenant: str, document_class, **kwargs) -> object:
        document = document_class.objects(**kwargs, context__tenant=tenant).first()
        # document.to_proto()
        return document

    def update_document(self, db_name: str, document_class, id: str, **kwargs) -> object:
        document = document_class.objects(id=id).first()
        document_class.objects(id=document.id).first().update(**kwargs)
        document.reload()
        return document

    def update(self, document_instance, **kwargs):
        document_instance.update(**kwargs)
        document_instance.reload()
        return document_instance

    def delete(self, document_instance):
        document_instance.delete()
        return document_instance

    def delete_document(self, db_name: str, document_class, id: str) -> object:
        document = document_class.objects(id=id).first()
        document.delete()
        return document

    def list_documents(
        self,
        db_name: str,
        tenant: str,
        document_class,
        fields: list = None,
        filter: dict = None,
        group_by: str = None,
        paginated: dict = None,
        sort_by: list = None,
    ) -> tuple[list, int]:
        """
        Parameters:
        fields (list): Optional list of fields to retrieve from the documents.
        filter (dict): Optional dictionary containing filter criteria for the query.
        group_by (str): Optional field to group results by.
        paginated (dict): Optional dictionary containing pagination information.
        sort_by (list): Optional list of fields to sort results by.

        Returns:
        list: A list of documents matching the specified criteria.
        """
        # Filter documents based on criteria provided

        if filter:
            query_set = document_class.objects(context__tenant=tenant).filter(__raw__=filter)
        else:
            query_set = document_class.objects(context__tenant=tenant)

        # Only retrieve specified fields
        if fields:
            query_set = query_set.only(*fields)

        # Group results by specified field
        if group_by:
            query_set = query_set.group_by(group_by)

        # Paginate results based on criteria provided
        if paginated:
            page = int(paginated.get("page") or 1)
            per_page = int(paginated.get("per_page") or 10)
            start = (page - 1) * per_page
            end = start + per_page
            query_set = query_set[start:end]

        # Sort results based on criteria provided
        if sort_by:
            query_set = query_set.order_by(*sort_by)

        # Return list of documents matching the specified criteria and total count of documents
        return list(query_set), query_set.count()

    def delete_documents(self, db_name, document_class, **kwargs):
        # with self.get_connection() as cnn:
        document = document_class.objects(**kwargs).delete()
        return document

    def update_embeded_document(
        self,
        db_name: str,
        document_class,
        filters: dict,
        update: dict,
        many: bool = False,
    ) -> object:
        # with self.get_connection() as cnn:
        if many:
            document_class.objects(**filters).update(**update)
            document = document_class.objects(**filters)
        else:
            document_class.objects(**filters).update_one(**update)
            document = document_class.objects(**filters).first()
        return document


class MongoConnection(object):
    """A MongoConnection class that can dynamically connect to a MongoDB database with MongoEngine and close the connection after each query.

    Args:
        host (str): The hostname or IP address of the MongoDB server.
        port (int): The port number of the MongoDB server.
        username (str): The username for the MongoDB database.
        password (str): The password for the MongoDB database.
        database (str): The name of the MongoDB database.
    """

    def __init__(self, host, port, db, username, password, complement):
        self.host = f"mongodb://{host}:{port}/?{'&'.join([f'{k}={v}' for (k, v) in complement.items()])}"
        self.port = port
        self.username = username
        self.password = password
        self.db = db

    def connect(self):
        """Connects to the MongoDB database.

        Returns:
            A MongoEngine connection object.
        """
        self.connection = mongo.connect(
            db=self.db,
            username=self.username,
            password=self.password,
            host=self.host,
        )
        return self.connection

    def close(self):
        """Closes the connection to the MongoDB database."""
        # self.connection.close()
        mongo.disconnect()

    def __enter__(self):
        """Enters a context manager.

        Returns:
            A MongoConnection object.
        """
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Exits a context manager."""
        self.close()


class SessionManager:
    def __init__(self, base_url):
        self.base_url = base_url
        self.engine = create_engine(self.base_url)
        self.session_factory = sessionmaker(bind=self.engine)
        self.Session = scoped_session(self.session_factory)

    def __enter__(self):
        # Esto dará una sesión específica para el hilo/contexto actual
        return self.Session()

    def __exit__(self, exc_type, exc_val, exc_tb):
        # Cuando el contexto se cierra, la sesión se cierra y elimina.
        if exc_tb is None:
            self.Session.commit()
        else:
            self.Session.rollback()
        self.Session.remove()


class PostgresDatabaseManager(SessionManager):
    """
    Handles database operations specific to Postgres using SQLAlchemy.
    Maneja operaciones de base de datos específicas para Postgres usando SQLAlchemy.

    Args:
    name (str): Name of the database.
                Nombre de la base de datos.
    host (str): Host of the database.
                Host de la base de datos.
    port (str): Port of the database.
                Puerto de la base de datos.
    user (str): User to connect to the database.
                Usuario para conectarse a la base de datos.
    password (str): Password to connect to the database.
                    Contraseña para conectarse a la base de datos.
    """

    DEFAULT_PAGE_SIZE = 10

    def __init__(self, name: str, host: str, port: str, user: str, password: str):
        """
        Initializes the PostgresDatabaseManager with the given database details.
        Inicializa el PostgresDatabaseManager con los detalles de base de datos proporcionados.

        Args:
        name (str): Name of the database.
                    Nombre de la base de datos.
        host (str): Host of the database.
                    Host de la base de datos.
        port (str): Port of the database.
                    Puerto de la base de datos.
        user (str): User to connect to the database.
                    Usuario para conectarse a la base de datos.
        password (str): Password to connect to the database.
                        Contraseña para conectarse a la base de datos.
        """
        self.name = name
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.base_url = f"postgresql://{user}:{password}@{host}:{port}/{name}"
        super().__init__(self.base_url)

    def get_db_connection(self):
        """
        Retrieves a connection to the Postgres database.
        Recupera una conexión a la base de datos Postgres.

        Returns:
        (obj): Database connection object.
               Objeto de conexión a la base de datos.
        """
        return self.engine.connect()

    def create_new_record(self, model, session, **kwargs):
        """
        Creates a new record in the database using the provided model and session.
        Crea un nuevo registro en la base de datos usando el modelo y la sesión proporcionados.

        Args:
        model (Base): The SQLAlchemy model where the record will be created.
                      El modelo SQLAlchemy donde se creará el registro.
        session (Session): An instance of the database session, likely from SQLAlchemy.
                           Una instancia de la sesión de base de datos, probablemente de SQLAlchemy.
        **kwargs: Attributes and values that will be used to initialize the new record of the model.
                  Atributos y valores que se utilizarán para inicializar el nuevo registro del modelo.

        Returns:
        (obj): The new model record after being added to the session.
               El nuevo registro del modelo después de ser añadido a la sesión.
        """
        record = model(**kwargs)
        record.create(session)
        return record

    def retrieve_record(self, model, session, filters: dict):
        """
        Retrieves a single database record based on provided filters.
        Recupera un único registro de la base de datos basado en los filtros proporcionados.

        Args:
        model (Base): The SQLAlchemy model to query.
                      El modelo SQLAlchemy a consultar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        filters (dict): A dictionary of attributes to filter the records by.
                        Un diccionario de atributos para filtrar los registros.

        Returns:
        (obj): The first record that matches the filters, or None if not found.
               El primer registro que coincida con los filtros, o None si no se encuentra.
        """
        return session.query(model).filter_by(**filters).first()

    def retrieve_records(self, model, session, filters: dict):
        """
        Retrieves database records based on provided filters.

        Args:
        model (Base): The SQLAlchemy model to query.
        session (Session): An instance of the database session.
        filters (dict): A dictionary of attributes to filter the records by.

        Returns:
        (objs): List of records that match with filters, or empty list if not found.
        """
        return session.query(model).filter_by(**filters).all()

    def retrieve_record_by_id(self, model, session, id: int):
        """
        Retrieves a single database record by its ID.
        Recupera un único registro de la base de datos por su ID.

        Args:
        model (Base): The SQLAlchemy model to query.
                      El modelo SQLAlchemy a consultar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        id (int): The ID of the record to retrieve.
                  El ID del registro a recuperar.

        Returns:
        (obj): The record with the given ID, or None if not found.
               El registro con el ID dado, o None si no se encuentra.
        """
        return session.query(model).get(id)

    def list_records(
        self,
        model,
        session,
        id: int,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
        paginated: base_pb2.Paginated,
    ):
        """
        Lists database records based on provided parameters.
        Lista registros de la base de datos basados en los parámetros proporcionados.

        Args:
        model (Base): The SQLAlchemy model to query.
                      El modelo SQLAlchemy a consultar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        id (int): The ID of a specific record to retrieve.
                  El ID de un registro específico para recuperar.
        fields (base_pb2.Fields): Fields to be returned in the response.
                                  Campos a ser devueltos en la respuesta.
        filter (base_pb2.Filter): Conditions to filter the list of records.
                                  Condiciones para filtrar la lista de registros.
        group_by (base_pb2.GroupBy): Conditions to group the list of records.
                                     Condiciones para agrupar la lista de registros.
        sort_by (base_pb2.SortBy): Conditions to sort the list of records.
                                   Condiciones para ordenar la lista de registros.
        paginated (base_pb2.Paginated): Conditions for pagination of the records.
                                        Condiciones para la paginación de los registros.

        Returns:
        (list): A list of records based on the provided parameters.
                Una lista de registros basados en los parámetros proporcionados.
        """
        # records = QueryBuilder.build_filter(model, session, id, fields, filter, group_by, sort_by, paginated)

        # Uso de la clase
        query = session.query(model)

        if filter.ListFields():
            # Uso de la clase
            expression = ast.literal_eval(filter.filter)  # Tu expresión en notación polaca inversa
            converter = PolishNotationToSQLAlchemy(model, expression)
            filter_condition, aliases = converter.convert()

            # Aplicar el filtro a la consulta
            for alias in aliases.values():
                query = query.join(alias)

            query = query.filter(filter_condition)

        if id:
            query = query.filter(model.id == id)

        if fields.ListFields():
            query = query.with_entities(*[getattr(model, f) for f in fields.name_field])

        if sort_by.ListFields():
            order_by_fields = self.build_sort_by(model, sort_by)
            query = query.order_by(*order_by_fields)

        total = query.count()

        page = paginated.offset or 1
        limit = paginated.limit or self.DEFAULT_PAGE_SIZE
        offset = (page - 1) * limit
        query = query.offset(offset).limit(limit)

        return query.all(), total

    def build_sort_by(self, model, sort_by: base_pb2.SortBy):
        field = getattr(model, sort_by.name_field)
        if sort_by.type == sort_by.DESC:
            return [desc(field)]
        return [asc(field)]

    def update_record(self, model, session, model_id, update_dict):
        """
        Update a database record of the given model with the specified changes.
        Actualiza un registro de base de datos del modelo dado con los cambios especificados.

        Args:
        model (Base): The SQLAlchemy model to update.
                      El modelo SQLAlchemy a actualizar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        model_id: The ID of the record to update.
                  El ID del registro a actualizar.
        update_dict (dict): A dictionary of attributes and their new values.
                            Un diccionario de atributos y sus nuevos valores.

        Returns:
        (obj): The updated record, or None if not found.
               El registro actualizado, o None si no se encuentra.
        """
        record = session.query(model).get(model_id)
        if not record:
            return None

        for key, value in update_dict.items():
            setattr(record, key, value)

        record.update(session)
        return record

    def delete_record_by_id(self, model, session, model_id):
        """
        Delete a database record of the given model by its ID.
        Elimina un registro de base de datos del modelo dado por su ID.

        Args:
        model (Base): The SQLAlchemy model to delete from.
                      El modelo SQLAlchemy del que eliminar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        model_id: The ID of the record to delete.
                  El ID del registro a eliminar.

        Returns:
        (bool): True if the record was deleted, False otherwise.
                Verdadero si el registro fue eliminado, Falso en caso contrario.
        """
        record = session.query(model).filter_by(id=model_id).first()
        if record:
            success = record.delete(session)
            if success:
                return True
        return False


class RedisConnection:
    def __init__(self, host: str, port: int, db: int) -> None:
        self.host = host
        self.port = int(port)
        self.db = db

    def __enter__(self) -> redis.StrictRedis:
        self.redis_client = redis.StrictRedis(host=self.host, port=self.port, db=self.db, decode_responses=True)
        return self.redis_client

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.redis_client.close()


class RedisManager(object):
    def __init__(self, host: str, port: int, db: int) -> None:
        self.host = host
        self.port = int(port)
        self.db = db
        self._connection = RedisConnection(host=self.host, port=self.port, db=self.db)

    def get_connection(self) -> RedisConnection:
        if Config.TESTING:
            return fakeredis.FakeStrictRedis(
                server=FakeRedisServer.get_instance(),
                charset="utf-8",
                decode_responses=True,
            )
        return self._connection

    def set_connection(self, connection: RedisConnection) -> None:
        self._connection = connection

    def set_json(self, key, json_obj):
        with self.get_connection() as rc:
            if isinstance(json_obj, str):
                json_obj = json.loads(json_obj)
            return rc.json().set(key, "$", json_obj)

    def get_json(self, key, *args, no_escape=False):
        with self.get_connection() as rc:
            return rc.json().get(key, *args, no_escape=no_escape)

    def get_resource_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_json(tenant_code)
        # logger.info(f"Redis config", extra={"deb_config": config})
        return {
            **nested(config, f"resources.{service_id}", {}),
            **nested(config, "aws", {}),
        }

    def get_aws_cognito_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "region_name": nested(config, "aws.cognito.region"),
            "aws_access_key_id": config.get("aws_access_key_id"),
            "aws_secret_access_key": config.get("aws_secret_access_key"),
            "user_pool_id": nested(config, "aws.cognito.user_pool_id"),
            "client_id": nested(config, "aws.cognito.client_id"),
        }

    def get_mongodb_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "host": nested(config, "dbs.mongodb.host"),
            "port": nested(config, "dbs.mongodb.port"),
            "user": nested(config, "dbs.mongodb.user"),
            "password": nested(config, "dbs.mongodb.pass"),
            "name": nested(config, "dbs.mongodb.name"),
            "complement": nested(config, "dbs.mongodb.complement"),
        }

    def get_postgres_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "host": nested(config, "dbs.postgres.host"),
            "port": nested(config, "dbs.postgres.port"),
            "user": nested(config, "dbs.postgres.user"),
            "password": nested(config, "dbs.postgres.pass"),
            "name": nested(config, "dbs.postgres.name"),
        }

    def get_tenant_codes(self, pattern="*") -> list:
        with self.get_connection() as rc:
            return rc.keys(pattern=pattern)

    def get_user_admin(self, tenant):
        tenant_obj = self.get_json(tenant)
        return tenant_obj.get("user_admin") or {}

    def get_load_balancer_config(self, service_id, tennat):
        config = self.get_resource_config(service_id, tennat)
        return {
            "host": nested(config, "load_balancer"),
            "port": nested(config, "port"),
        }

    def get_load_balancer_name(self, service_id, tennat):
        config = self.get_load_balancer_config(service_id, tennat)
        return f"{config.get('host')}:{config.get('port')}"


class PolishNotationToMongoDB:
    def __init__(self, expression):
        self.expression = expression
        self.operators_logical = {
            "and": "$and",
            "or": "$or",
            "nor": "$nor",
            "not": "$not",
        }
        self.operators_comparison = {
            "=": "$eq",
            ">": "$gt",
            "<": "$lt",
            ">=": "$gte",
            "<=": "$lte",
            "in": "$in",
            "nin": "$nin",
            "!=": "$ne",
            "!like": "$not",
            "like": "$regex",
        }

    def is_logical_operator(self, token):
        if not isinstance(token, str):
            return False
        return token in self.operators_logical

    def is_comparison_operator(self, token):
        if not isinstance(token, str):
            return False
        return token in self.operators_comparison

    def is_tuple(self, token):
        return isinstance(token, tuple) and len(token) == 3

    def convert(self):
        operand_stack = []
        operator_stack = []

        for token in reversed(self.expression):
            if self.is_logical_operator(token):
                operator_stack.append(token)
            elif self.is_comparison_operator(token):
                operator_stack.append(token)
            elif self.is_tuple(token):
                field, old_operator, value = token
                if old_operator in self.operators_comparison:
                    options = {}
                    if old_operator == "like":
                        options = {"$options": "i"}
                    elif old_operator == "!like":
                        options = {
                            self.operators_comparison[old_operator]: {
                                "$regex": value,
                                "$options": "i",
                            }
                        }
                    operand_stack.append({field: {self.operators_comparison[old_operator]: value} | options})
                else:
                    raise ValueError(f"Unexpected operator: {old_operator}")
            else:
                raise ValueError(f"Unexpected token: {token}")

        while operator_stack:
            operator = operator_stack.pop()
            if operator in self.operators_logical:
                operands = []
                for _ in range(2):
                    operands.append(operand_stack.pop())
                operand_stack.append({self.operators_logical[operator]: operands})
            else:
                raise ValueError(f"Unexpected operator: {operator}")

        return operand_stack.pop()


class DBUtil(object):
    @classmethod
    def db_prepared_statement(
        cls,
        id: str,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        paginated: base_pb2.Paginated,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
    ) -> dict:
        prepared_statement = {}
        prepared_statement["paginated"] = {
            "page": paginated.offset,
            "per_page": paginated.limit or 10,
        }
        if (ft := filter.ListFields()) or id:
            expression = [("_id", "=", cls.generate_object_id(id))]
            if ft:
                str_filter = filter.filter.replace("true", "True").replace("false", "False").replace("__", ".")
                expression = ast.literal_eval(str_filter)
                # reemplace filter id by _id and convert to ObjectId
                for idx, exp in enumerate(expression):
                    if isinstance(exp, tuple) and len(exp) == 3 and exp[0] == "id":
                        if type(exp[2]) == list:
                            expression[idx] = (
                                "_id",
                                exp[1],
                                [cls.generate_object_id(x) for x in exp[2]],
                            )
                            continue
                        expression[idx] = (
                            "_id",
                            exp[1],
                            cls.generate_object_id(exp[2]),
                        )
            filter_custom = PolishNotationToMongoDB(expression=expression).convert()
            prepared_statement["filter"] = filter_custom
        if group_by:
            prepared_statement["group_by"] = [x.name_field for x in group_by]
        if sort_by.ListFields():
            prepared_statement["sort_by"] = [cls.db_trans_sort(sort_by)]
        if fields:
            prepared_statement["fields"] = fields.name_field
        return prepared_statement

    @classmethod
    def db_trans_sort(cls, sort_by: base_pb2.SortBy) -> str:
        if not sort_by.name_field:
            return None
        return f"{'-' if sort_by.type == sort_by.DESC else '+'}{sort_by.name_field}"

    @classmethod
    def generate_object_id(cls, id=None):
        try:
            return ObjectId(id)
        except:
            return ObjectId(None)


class PolishNotationToSQLAlchemy:
    def __init__(self, model, expression):
        self.model = model
        self.expression = expression
        self.aliases = {}

    def is_logical_operator(self, token):
        return token in ["and", "or", "not"]

    def is_comparison_operator(self, token):
        return token in ["=", "!=", "like", "!like", "in", "nin", ">", "<", ">=", "<="]

    def is_tuple(self, token):
        return isinstance(token, tuple) and len(token) == 3

    def get_field(self, model, field_path):
        fields = field_path.split("__")
        for field in fields[:-1]:  # process relationships
            relationship = getattr(model, field, None)
            if relationship is None:
                raise AttributeError(f"No such relationship {field} on {model}")
            if relationship in self.aliases:
                model = self.aliases[relationship]
            else:
                alias = aliased(relationship.property.mapper.class_)
                self.aliases[relationship] = alias
                model = alias
        return getattr(model, fields[-1], None), model

    def create_filter(self, model, field_path, operator, value):
        field, model = self.get_field(model, field_path)
        if operator == "=":
            return field == value
        elif operator == "!=":
            return field != value
        elif operator == "like":
            return field.like(value)
        elif operator == "!like":
            return not_(field.like(value))
        elif operator == "in":
            return field.in_(value)
        elif operator == "nin":
            return not_(field.in_(value))
        elif operator == ">":
            return field > value
        elif operator == "<":
            return field < value
        elif operator == ">=":
            return field >= value
        elif operator == "<=":
            return field <= value
        else:
            raise ValueError(f"Unknown operator: {operator}")

    def convert(self):
        operand_stack = []

        for token in reversed(self.expression):
            if self.is_logical_operator(token):
                operands = [operand_stack.pop() for _ in range(2)]
                if token == "and":
                    operand_stack.append(and_(*operands))
                elif token == "or":
                    operand_stack.append(or_(*operands))
                elif token == "not":
                    operand_stack.append(not_(operands[0]))
            elif self.is_tuple(token):
                field_path, operator, value = token
                operand_stack.append(self.create_filter(self.model, field_path, operator, value))
            else:
                raise ValueError(f"Unexpected token: {token}")

        if len(operand_stack) != 1:
            raise ValueError("The expression does not resolve to a single operand.")

        return operand_stack[0], self.aliases


class QueryBuilder:
    filter_ops = {
        "=": operator.eq,
        "<": operator.lt,
        "<=": operator.le,
        ">": operator.gt,
        ">=": operator.ge,
        "!=": operator.ne,
        "in": lambda x, y: x.in_(y),
        "and": operator.and_,
        "or": operator.or_,
        "not": lambda x: ~(x),
    }
    DEFAULT_PAGE_SIZE = 10

    @classmethod
    def pre_to_in(cls, filters: base_pb2.Filter) -> list:
        str_filter = filters.filter.replace("true", "True").replace("false", "False")
        expression = ast.literal_eval(str_filter)
        stack = []
        result = []
        for item in expression:
            if isinstance(item, tuple):
                if stack:
                    result.append(item)
                    result.append(stack.pop())
                else:
                    result.append(item)
            else:
                stack.append(item)
        return result

    @classmethod
    def build_filter(
        cls,
        model,
        session,
        id: int,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
        paginated: base_pb2.Paginated,
    ):
        query = session.query(model)

        if id:
            query = query.filter(model.id == id)

        if fields.ListFields():
            query = query.with_entities(*[getattr(model, f) for f in fields.name_field])

        if filter.ListFields():
            filter_custom = cls.pre_to_in(filter)
            query_filter = cls.build_query(model, filter_custom)
            query = query.filter(query_filter)

        if group_by:
            group_by_fields = cls.build_group_by(model, group_by)
            query = query.group_by(*group_by_fields)

        if sort_by.ListFields():
            order_by_fields = cls.build_sort_by(model, sort_by)
            query = query.order_by(*order_by_fields)

        """
            Retrieves a paginated list from a given query.

            Args:
            
            query (obj): The query object that needs pagination.
            paginated (obj): Contains pagination parameters such as offset and limit.

            Returns:
            
            tuple: A tuple containing the paginated list and the total count.
            
        """

        total = query.count()

        if paginated.ListFields():
            page = paginated.offset or 1
            limit = paginated.limit or cls.DEFAULT_PAGE_SIZE
            offset = (page - 1) * limit
            query = query.offset(offset).limit(limit)

        return query.all(), total

    @classmethod
    def build_query(cls, model, filters: list):
        query = None

        filter_operator = None
        for item in filters:
            if isinstance(item, str):
                filter_operator = item.lower()
                if filter_operator not in cls.filter_ops:
                    raise ValueError(f"Invalid filter operator: {filter_operator}")
            elif isinstance(item, tuple) and len(item) == 3:
                field, op, value = item
                if "." in field:
                    related, field = field.split(".")
                    related_model = getattr(model, related).property.argument
                    related_field = getattr(related_model, field)
                    related_query = cls.filter_ops[op](related_field, value)
                    query = cls.filter_ops[op](query, related_query) if query else related_query
                else:
                    field = getattr(model, field)
                    field_query = cls.filter_ops[op](field, value)
                    query = cls.filter_ops[filter_operator](query, field_query) if filter_operator else field_query
            else:
                raise ValueError(f"Invalid filter item: {item}")

        return query

    @classmethod
    def build_sort_by(cls, model, sort_by: base_pb2.SortBy):
        field = getattr(model, sort_by.name_field)
        if sort_by.type == sort_by.DESC:
            return [desc(field)]
        return [asc(field)]


class FakeRedisServer:
    _instance = None

    @classmethod
    def get_instance(cls) -> fakeredis.FakeServer:
        if not cls._instance:
            cls._instance = cls._create_instance()
        return cls._instance

    @classmethod
    def _create_instance(cls) -> fakeredis.FakeServer:
        server = fakeredis.FakeServer()
        return server



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/users/user.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x13v1/users/user.proto\x12\x1epro.omni.oms.api.v1.users.user\x1a\x11\x63ommon/base.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1egoogle/protobuf/wrappers.proto"\xec\x03\n\x04User\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0b\n\x03sub\x18\x03 \x01(\t\x12\x0e\n\x06tenant\x18\x04 \x01(\t\x12\x10\n\x08username\x18\x05 \x01(\t\x12\r\n\x05\x65mail\x18\x06 \x01(\t\x12\x36\n\x08language\x18\x07 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Object\x12\x36\n\x08timezone\x18\x08 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Object\x12*\n\x06groups\x18\t \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12\x30\n\x0cis_superuser\x18\n \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12*\n\x06\x61\x63tive\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\'\n\x03mfa\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\r \x01(\t\x12?\n\x0cobject_audit\x18\x0e \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit\x12\x13\n\x0bpermissions\x18\x0f \x03(\t"\xdd\x01\n\x05Group\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12*\n\x06\x61\x63\x63\x65ss\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12*\n\x06\x61\x63tive\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x06 \x01(\t\x12?\n\x0cobject_audit\x18\x07 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xec\x01\n\x06\x41\x63tion\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x14\n\x0cmicroservice\x18\x05 \x01(\t\x12\r\n\x05model\x18\x06 \x01(\t\x12*\n\x06\x61\x63tive\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x08 \x01(\t\x12?\n\x0cobject_audit\x18\t \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\x8d\x02\n\x06\x41\x63\x63\x65ss\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12\x0e\n\x06\x64omain\x18\x04 \x01(\t\x12\x11\n\taction_id\x18\x05 \x01(\t\x12\x36\n\x06\x61\x63tion\x18\x06 \x01(\x0b\x32&.pro.omni.oms.api.v1.users.user.Action\x12*\n\x06\x61\x63tive\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x08 \x01(\t\x12?\n\x0cobject_audit\x18\t \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xa4\x03\n\x11UserCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\r\n\x05\x65mail\x18\x03 \x01(\t\x12\x15\n\remail_confirm\x18\x04 \x01(\t\x12\x10\n\x08password\x18\x05 \x01(\t\x12\x18\n\x10password_confirm\x18\x06 \x01(\t\x12.\n\ngroups_ids\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12\x30\n\x0cis_superuser\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x36\n\x08language\x18\t \x01(\x0b\x32$.pro.omni.oms.api.common.base.Object\x12\x36\n\x08timezone\x18\n \x01(\x0b\x32$.pro.omni.oms.api.common.base.Object\x12\x13\n\x0b\x65xternal_id\x18\x0b \x01(\t\x12\x36\n\x07\x63ontext\x18\x0c \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x93\x01\n\x12UserCreateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x32\n\x04user\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.v1.users.user.User"\xed\x02\n\x0fUserReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xcd\x01\n\x10UserReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12\x33\n\x05users\x18\x03 \x03(\x0b\x32$.pro.omni.oms.api.v1.users.user.User"\x7f\n\x11UserUpdateRequest\x12\x32\n\x04user\x18\x01 \x01(\x0b\x32$.pro.omni.oms.api.v1.users.user.User\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x93\x01\n\x12UserUpdateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x32\n\x04user\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.v1.users.user.User"W\n\x11UserDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"_\n\x12UserDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x8b\x01\n\x19UserChangePasswordRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\x18\n\x10password_confirm\x18\x03 \x01(\t\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"g\n\x1aUserChangePasswordResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x82\x01\n\x16UserChangeEmailRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05\x65mail\x18\x02 \x01(\t\x12\x15\n\remail_confirm\x18\x03 \x01(\t\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"d\n\x17UserChangeEmailResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x98\x01\n\x12GroupCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12.\n\naccess_ids\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x96\x01\n\x13GroupCreateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x34\n\x05group\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.v1.users.user.Group"\xee\x02\n\x10GroupReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd0\x01\n\x11GroupReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12\x35\n\x06groups\x18\x03 \x03(\x0b\x32%.pro.omni.oms.api.v1.users.user.Group"\x82\x01\n\x12GroupUpdateRequest\x12\x34\n\x05group\x18\x01 \x01(\x0b\x32%.pro.omni.oms.api.v1.users.user.Group\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x96\x01\n\x13GroupUpdateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x34\n\x05group\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.v1.users.user.Group"X\n\x12GroupDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"`\n\x13GroupDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xb8\x01\n\x13\x41\x63tionCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x14\n\x0cmicroservice\x18\x04 \x01(\t\x12\r\n\x05model\x18\x05 \x01(\t\x12\x13\n\x0b\x65xternal_id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x99\x01\n\x14\x41\x63tionCreateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x36\n\x06\x61\x63tion\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.v1.users.user.Action"\xef\x02\n\x11\x41\x63tionReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd3\x01\n\x12\x41\x63tionReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12\x37\n\x07\x61\x63tions\x18\x03 \x03(\x0b\x32&.pro.omni.oms.api.v1.users.user.Action"\x85\x01\n\x13\x41\x63tionUpdateRequest\x12\x36\n\x06\x61\x63tion\x18\x01 \x01(\x0b\x32&.pro.omni.oms.api.v1.users.user.Action\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x99\x01\n\x14\x41\x63tionUpdateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x36\n\x06\x61\x63tion\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.v1.users.user.Action"Y\n\x13\x41\x63tionDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"a\n\x14\x41\x63tionDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xa1\x01\n\x13\x41\x63\x63\x65ssCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x0e\n\x06\x64omain\x18\x03 \x01(\t\x12\x11\n\taction_id\x18\x04 \x01(\t\x12\x13\n\x0b\x65xternal_id\x18\x05 \x01(\t\x12\x36\n\x07\x63ontext\x18\x06 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x99\x01\n\x14\x41\x63\x63\x65ssCreateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x36\n\x06\x61\x63\x63\x65ss\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.v1.users.user.Access"\xef\x02\n\x11\x41\x63\x63\x65ssReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd4\x01\n\x12\x41\x63\x63\x65ssReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12\x38\n\x08\x61\x63\x63\x65sses\x18\x03 \x03(\x0b\x32&.pro.omni.oms.api.v1.users.user.Access"\x85\x01\n\x13\x41\x63\x63\x65ssUpdateRequest\x12\x36\n\x06\x61\x63\x63\x65ss\x18\x01 \x01(\x0b\x32&.pro.omni.oms.api.v1.users.user.Access\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x99\x01\n\x14\x41\x63\x63\x65ssUpdateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x36\n\x06\x61\x63\x63\x65ss\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.v1.users.user.Access"Y\n\x13\x41\x63\x63\x65ssDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"a\n\x14\x41\x63\x63\x65ssDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xa3\x01\n\x12\x41\x63\x63\x65ssGroupRequest\x12.\n\naccess_ids\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12\x10\n\x08group_id\x18\x02 \x01(\t\x12\x13\n\x0b\x65xternal_id\x18\x03 \x01(\t\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"`\n\x13\x41\x63\x63\x65ssGroupResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x8a\x01\n\x10GroupUserRequest\x12-\n\tgroup_ids\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"^\n\x11GroupUserResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"t\n\x14HasPermissionRequest\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x12\n\npermission\x18\x02 \x01(\t\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xae\x01\n\x15HasPermissionResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x16\n\x0ehas_permission\x18\x02 \x01(\x08\x12\x32\n\x04user\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.v1.users.user.User"j\n\x0cLoginRequest\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"P\n\x14\x41uthenticationResult\x12\r\n\x05token\x18\x01 \x01(\t\x12\x15\n\rrefresh_token\x18\x02 \x01(\t\x12\x12\n\nexpires_in\x18\x03 \x01(\x05"\xe3\x01\n\rLoginResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12S\n\x15\x61uthentication_result\x18\x02 \x01(\x0b\x32\x34.pro.omni.oms.api.v1.users.user.AuthenticationResult\x12\x32\n\x04user\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.v1.users.user.User"d\n\x13RefreshTokenRequest\x12\x36\n\x07\x63ontext\x18\x01 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context\x12\x15\n\rrefresh_token\x18\x02 \x01(\t"\xb6\x01\n\x14RefreshTokenResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12S\n\x15\x61uthentication_result\x18\x02 \x01(\x0b\x32\x34.pro.omni.oms.api.v1.users.user.AuthenticationResult2\x99\x18\n\x0cUsersService\x12u\n\nUserCreate\x12\x31.pro.omni.oms.api.v1.users.user.UserCreateRequest\x1a\x32.pro.omni.oms.api.v1.users.user.UserCreateResponse"\x00\x12o\n\x08UserRead\x12/.pro.omni.oms.api.v1.users.user.UserReadRequest\x1a\x30.pro.omni.oms.api.v1.users.user.UserReadResponse"\x00\x12u\n\nUserUpdate\x12\x31.pro.omni.oms.api.v1.users.user.UserUpdateRequest\x1a\x32.pro.omni.oms.api.v1.users.user.UserUpdateResponse"\x00\x12u\n\nUserDelete\x12\x31.pro.omni.oms.api.v1.users.user.UserDeleteRequest\x1a\x32.pro.omni.oms.api.v1.users.user.UserDeleteResponse"\x00\x12\x8d\x01\n\x12UserChangePassword\x12\x39.pro.omni.oms.api.v1.users.user.UserChangePasswordRequest\x1a:.pro.omni.oms.api.v1.users.user.UserChangePasswordResponse"\x00\x12\x84\x01\n\x0fUserChangeEmail\x12\x36.pro.omni.oms.api.v1.users.user.UserChangeEmailRequest\x1a\x37.pro.omni.oms.api.v1.users.user.UserChangeEmailResponse"\x00\x12x\n\x0bGroupCreate\x12\x32.pro.omni.oms.api.v1.users.user.GroupCreateRequest\x1a\x33.pro.omni.oms.api.v1.users.user.GroupCreateResponse"\x00\x12r\n\tGroupRead\x12\x30.pro.omni.oms.api.v1.users.user.GroupReadRequest\x1a\x31.pro.omni.oms.api.v1.users.user.GroupReadResponse"\x00\x12x\n\x0bGroupUpdate\x12\x32.pro.omni.oms.api.v1.users.user.GroupUpdateRequest\x1a\x33.pro.omni.oms.api.v1.users.user.GroupUpdateResponse"\x00\x12x\n\x0bGroupDelete\x12\x32.pro.omni.oms.api.v1.users.user.GroupDeleteRequest\x1a\x33.pro.omni.oms.api.v1.users.user.GroupDeleteResponse"\x00\x12{\n\x0c\x41\x63tionCreate\x12\x33.pro.omni.oms.api.v1.users.user.ActionCreateRequest\x1a\x34.pro.omni.oms.api.v1.users.user.ActionCreateResponse"\x00\x12u\n\nActionRead\x12\x31.pro.omni.oms.api.v1.users.user.ActionReadRequest\x1a\x32.pro.omni.oms.api.v1.users.user.ActionReadResponse"\x00\x12{\n\x0c\x41\x63tionUpdate\x12\x33.pro.omni.oms.api.v1.users.user.ActionUpdateRequest\x1a\x34.pro.omni.oms.api.v1.users.user.ActionUpdateResponse"\x00\x12{\n\x0c\x41\x63tionDelete\x12\x33.pro.omni.oms.api.v1.users.user.ActionDeleteRequest\x1a\x34.pro.omni.oms.api.v1.users.user.ActionDeleteResponse"\x00\x12{\n\x0c\x41\x63\x63\x65ssCreate\x12\x33.pro.omni.oms.api.v1.users.user.AccessCreateRequest\x1a\x34.pro.omni.oms.api.v1.users.user.AccessCreateResponse"\x00\x12u\n\nAccessRead\x12\x31.pro.omni.oms.api.v1.users.user.AccessReadRequest\x1a\x32.pro.omni.oms.api.v1.users.user.AccessReadResponse"\x00\x12{\n\x0c\x41\x63\x63\x65ssUpdate\x12\x33.pro.omni.oms.api.v1.users.user.AccessUpdateRequest\x1a\x34.pro.omni.oms.api.v1.users.user.AccessUpdateResponse"\x00\x12{\n\x0c\x41\x63\x63\x65ssDelete\x12\x33.pro.omni.oms.api.v1.users.user.AccessDeleteRequest\x1a\x34.pro.omni.oms.api.v1.users.user.AccessDeleteResponse"\x00\x12~\n\rHasPermission\x12\x34.pro.omni.oms.api.v1.users.user.HasPermissionRequest\x1a\x35.pro.omni.oms.api.v1.users.user.HasPermissionResponse"\x00\x12\x66\n\x05Login\x12,.pro.omni.oms.api.v1.users.user.LoginRequest\x1a-.pro.omni.oms.api.v1.users.user.LoginResponse"\x00\x12{\n\x0cRefreshToken\x12\x33.pro.omni.oms.api.v1.users.user.RefreshTokenRequest\x1a\x34.pro.omni.oms.api.v1.users.user.RefreshTokenResponse"\x00\x12}\n\x10\x41\x64\x64\x41\x63\x63\x65ssToGroup\x12\x32.pro.omni.oms.api.v1.users.user.AccessGroupRequest\x1a\x33.pro.omni.oms.api.v1.users.user.AccessGroupResponse"\x00\x12\x82\x01\n\x15RemoveAccessFromGroup\x12\x32.pro.omni.oms.api.v1.users.user.AccessGroupRequest\x1a\x33.pro.omni.oms.api.v1.users.user.AccessGroupResponse"\x00\x12w\n\x0e\x41\x64\x64GroupToUser\x12\x30.pro.omni.oms.api.v1.users.user.GroupUserRequest\x1a\x31.pro.omni.oms.api.v1.users.user.GroupUserResponse"\x00\x12|\n\x13RemoveGroupFromUser\x12\x30.pro.omni.oms.api.v1.users.user.GroupUserRequest\x1a\x31.pro.omni.oms.api.v1.users.user.GroupUserResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.users.user_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_USER"]._serialized_start = 137
    _globals["_USER"]._serialized_end = 629
    _globals["_GROUP"]._serialized_start = 632
    _globals["_GROUP"]._serialized_end = 853
    _globals["_ACTION"]._serialized_start = 856
    _globals["_ACTION"]._serialized_end = 1092
    _globals["_ACCESS"]._serialized_start = 1095
    _globals["_ACCESS"]._serialized_end = 1364
    _globals["_USERCREATEREQUEST"]._serialized_start = 1367
    _globals["_USERCREATEREQUEST"]._serialized_end = 1787
    _globals["_USERCREATERESPONSE"]._serialized_start = 1790
    _globals["_USERCREATERESPONSE"]._serialized_end = 1937
    _globals["_USERREADREQUEST"]._serialized_start = 1940
    _globals["_USERREADREQUEST"]._serialized_end = 2305
    _globals["_USERREADRESPONSE"]._serialized_start = 2308
    _globals["_USERREADRESPONSE"]._serialized_end = 2513
    _globals["_USERUPDATEREQUEST"]._serialized_start = 2515
    _globals["_USERUPDATEREQUEST"]._serialized_end = 2642
    _globals["_USERUPDATERESPONSE"]._serialized_start = 2645
    _globals["_USERUPDATERESPONSE"]._serialized_end = 2792
    _globals["_USERDELETEREQUEST"]._serialized_start = 2794
    _globals["_USERDELETEREQUEST"]._serialized_end = 2881
    _globals["_USERDELETERESPONSE"]._serialized_start = 2883
    _globals["_USERDELETERESPONSE"]._serialized_end = 2978
    _globals["_USERCHANGEPASSWORDREQUEST"]._serialized_start = 2981
    _globals["_USERCHANGEPASSWORDREQUEST"]._serialized_end = 3120
    _globals["_USERCHANGEPASSWORDRESPONSE"]._serialized_start = 3122
    _globals["_USERCHANGEPASSWORDRESPONSE"]._serialized_end = 3225
    _globals["_USERCHANGEEMAILREQUEST"]._serialized_start = 3228
    _globals["_USERCHANGEEMAILREQUEST"]._serialized_end = 3358
    _globals["_USERCHANGEEMAILRESPONSE"]._serialized_start = 3360
    _globals["_USERCHANGEEMAILRESPONSE"]._serialized_end = 3460
    _globals["_GROUPCREATEREQUEST"]._serialized_start = 3463
    _globals["_GROUPCREATEREQUEST"]._serialized_end = 3615
    _globals["_GROUPCREATERESPONSE"]._serialized_start = 3618
    _globals["_GROUPCREATERESPONSE"]._serialized_end = 3768
    _globals["_GROUPREADREQUEST"]._serialized_start = 3771
    _globals["_GROUPREADREQUEST"]._serialized_end = 4137
    _globals["_GROUPREADRESPONSE"]._serialized_start = 4140
    _globals["_GROUPREADRESPONSE"]._serialized_end = 4348
    _globals["_GROUPUPDATEREQUEST"]._serialized_start = 4351
    _globals["_GROUPUPDATEREQUEST"]._serialized_end = 4481
    _globals["_GROUPUPDATERESPONSE"]._serialized_start = 4484
    _globals["_GROUPUPDATERESPONSE"]._serialized_end = 4634
    _globals["_GROUPDELETEREQUEST"]._serialized_start = 4636
    _globals["_GROUPDELETEREQUEST"]._serialized_end = 4724
    _globals["_GROUPDELETERESPONSE"]._serialized_start = 4726
    _globals["_GROUPDELETERESPONSE"]._serialized_end = 4822
    _globals["_ACTIONCREATEREQUEST"]._serialized_start = 4825
    _globals["_ACTIONCREATEREQUEST"]._serialized_end = 5009
    _globals["_ACTIONCREATERESPONSE"]._serialized_start = 5012
    _globals["_ACTIONCREATERESPONSE"]._serialized_end = 5165
    _globals["_ACTIONREADREQUEST"]._serialized_start = 5168
    _globals["_ACTIONREADREQUEST"]._serialized_end = 5535
    _globals["_ACTIONREADRESPONSE"]._serialized_start = 5538
    _globals["_ACTIONREADRESPONSE"]._serialized_end = 5749
    _globals["_ACTIONUPDATEREQUEST"]._serialized_start = 5752
    _globals["_ACTIONUPDATEREQUEST"]._serialized_end = 5885
    _globals["_ACTIONUPDATERESPONSE"]._serialized_start = 5888
    _globals["_ACTIONUPDATERESPONSE"]._serialized_end = 6041
    _globals["_ACTIONDELETEREQUEST"]._serialized_start = 6043
    _globals["_ACTIONDELETEREQUEST"]._serialized_end = 6132
    _globals["_ACTIONDELETERESPONSE"]._serialized_start = 6134
    _globals["_ACTIONDELETERESPONSE"]._serialized_end = 6231
    _globals["_ACCESSCREATEREQUEST"]._serialized_start = 6234
    _globals["_ACCESSCREATEREQUEST"]._serialized_end = 6395
    _globals["_ACCESSCREATERESPONSE"]._serialized_start = 6398
    _globals["_ACCESSCREATERESPONSE"]._serialized_end = 6551
    _globals["_ACCESSREADREQUEST"]._serialized_start = 6554
    _globals["_ACCESSREADREQUEST"]._serialized_end = 6921
    _globals["_ACCESSREADRESPONSE"]._serialized_start = 6924
    _globals["_ACCESSREADRESPONSE"]._serialized_end = 7136
    _globals["_ACCESSUPDATEREQUEST"]._serialized_start = 7139
    _globals["_ACCESSUPDATEREQUEST"]._serialized_end = 7272
    _globals["_ACCESSUPDATERESPONSE"]._serialized_start = 7275
    _globals["_ACCESSUPDATERESPONSE"]._serialized_end = 7428
    _globals["_ACCESSDELETEREQUEST"]._serialized_start = 7430
    _globals["_ACCESSDELETEREQUEST"]._serialized_end = 7519
    _globals["_ACCESSDELETERESPONSE"]._serialized_start = 7521
    _globals["_ACCESSDELETERESPONSE"]._serialized_end = 7618
    _globals["_ACCESSGROUPREQUEST"]._serialized_start = 7621
    _globals["_ACCESSGROUPREQUEST"]._serialized_end = 7784
    _globals["_ACCESSGROUPRESPONSE"]._serialized_start = 7786
    _globals["_ACCESSGROUPRESPONSE"]._serialized_end = 7882
    _globals["_GROUPUSERREQUEST"]._serialized_start = 7885
    _globals["_GROUPUSERREQUEST"]._serialized_end = 8023
    _globals["_GROUPUSERRESPONSE"]._serialized_start = 8025
    _globals["_GROUPUSERRESPONSE"]._serialized_end = 8119
    _globals["_HASPERMISSIONREQUEST"]._serialized_start = 8121
    _globals["_HASPERMISSIONREQUEST"]._serialized_end = 8237
    _globals["_HASPERMISSIONRESPONSE"]._serialized_start = 8240
    _globals["_HASPERMISSIONRESPONSE"]._serialized_end = 8414
    _globals["_LOGINREQUEST"]._serialized_start = 8416
    _globals["_LOGINREQUEST"]._serialized_end = 8522
    _globals["_AUTHENTICATIONRESULT"]._serialized_start = 8524
    _globals["_AUTHENTICATIONRESULT"]._serialized_end = 8604
    _globals["_LOGINRESPONSE"]._serialized_start = 8607
    _globals["_LOGINRESPONSE"]._serialized_end = 8834
    _globals["_REFRESHTOKENREQUEST"]._serialized_start = 8836
    _globals["_REFRESHTOKENREQUEST"]._serialized_end = 8936
    _globals["_REFRESHTOKENRESPONSE"]._serialized_start = 8939
    _globals["_REFRESHTOKENRESPONSE"]._serialized_end = 9121
    _globals["_USERSSERVICE"]._serialized_start = 9124
    _globals["_USERSSERVICE"]._serialized_end = 12221
# @@protoc_insertion_point(module_scope)



import importlib

import grpc
from grpc._channel import Channel
from grpc.experimental import _insecure_channel_credentials
from omni.pro.cloudmap import CloudMap
from omni.pro.config import Config
from omni.pro.logger import configure_logger
from omni.pro.protos.util import format_request
from omni.pro.database import RedisManager
from omni.pro.util import nested

logger = configure_logger(name=__name__)


class Event(dict):
    def __init__(
        self,
        module_grpc: str,
        stub_classname: str,
        rpc_method: str,
        module_pb2: str,
        request_class: str,
        params: dict = {},
    ):
        super().__init__()
        self["module_grpc"] = module_grpc
        self["stub_classname"] = stub_classname
        self["rpc_method"] = rpc_method
        self["module_pb2"] = module_pb2
        self["request_class"] = request_class
        self["params"] = params


class OmniChannel(Channel):
    def __init__(
        self,
        service_id,
        credentials=None,
        options=None,
        compression=None,
        tennat=None,
    ):
        if not Config.DEBUG:
            credentials = credentials or grpc.ssl_channel_credentials()
            if credentials is None or credentials._credentials is _insecure_channel_credentials:
                raise ValueError(
                    "secure_channel cannot be called with insecure credentials." + " Call insecure_channel instead."
                )
            credentials = credentials._credentials
            options = options or [("grpc.ssl_target_name_override", "omni.pro")]

        # cloud_map = CloudMap(service_name=Config.SERVICE_NAME_BALANCER)
        # target = cloud_map.get_url_channel(service_id)
        target = self.get_target(service_id, tennat)
        super().__init__(target, () if options is None else options, credentials, compression)

    def get_target(self, service_id, tennat):
        redis = RedisManager(host=Config.REDIS_HOST, port=Config.REDIS_PORT, db=Config.REDIS_DB)
        return redis.get_load_balancer_name(service_id, tennat)


class GRPClient(object):
    def __init__(self, service_id: str):
        self.service_id = service_id

    def call_rpc_fuction(self, event: Event, *args, **kwargs):
        """
        function to call rpc function
        :param event: Event with params to call rpc function
        ```
        event = Event(
            module_grpc="v1.users.user_pb2_grpc",
            stub_classname="UsersServiceStub",
            rpc_method="UserRead",
            module_pb2="v1.users.user_pb2",
            request_class="UserReadRequest",
            params={"id": "64adc0477be3ec5e9160b16e", "context": {"tenant": "SPLA", "user": "admin"}},
        )
        response, success = GRPClient(service_id=Config.SERVICE_ID).call_rpc_fuction(event)
        ```
        """
        with OmniChannel(self.service_id, *args, **kwargs, tennat=nested(event, "params.context.tenant")) as channel:
            stub = event.get("service_stub")
            stub_classname = event.get("stub_classname")
            path_module = "omni.pro.protos"
            module_grpc = importlib.import_module(f"{path_module}.{event.get('module_grpc')}")
            stub = getattr(module_grpc, stub_classname)(channel)
            request_class = event.get("request_class")
            module_pb2 = importlib.import_module(f"{path_module}.{event.get('module_pb2')}")
            request = format_request(event.get("params"), request_class, module_pb2)
            # Instance the method rpc que recibe el request
            response = getattr(stub, event.get("rpc_method"))(request)
            success = response.response_standard.status_code in range(200, 300)
            return response, success



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.utilities import webhook_pb2 as v1_dot_utilities_dot_webhook__pb2


class WebhookServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.WebhookCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookCreate",
            request_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookCreateRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookCreateResponse.FromString,
        )
        self.WebhookRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookRead",
            request_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookReadRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookReadResponse.FromString,
        )
        self.WebhookUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookUpdate",
            request_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookUpdateResponse.FromString,
        )
        self.WebhookDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookDelete",
            request_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookDeleteResponse.FromString,
        )


class WebhookServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def WebhookCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def WebhookRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def WebhookUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def WebhookDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_WebhookServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "WebhookCreate": grpc.unary_unary_rpc_method_handler(
            servicer.WebhookCreate,
            request_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookCreateRequest.FromString,
            response_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookCreateResponse.SerializeToString,
        ),
        "WebhookRead": grpc.unary_unary_rpc_method_handler(
            servicer.WebhookRead,
            request_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookReadRequest.FromString,
            response_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookReadResponse.SerializeToString,
        ),
        "WebhookUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.WebhookUpdate,
            request_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookUpdateRequest.FromString,
            response_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookUpdateResponse.SerializeToString,
        ),
        "WebhookDelete": grpc.unary_unary_rpc_method_handler(
            servicer.WebhookDelete,
            request_deserializer=v1_dot_utilities_dot_webhook__pb2.WebhookDeleteRequest.FromString,
            response_serializer=v1_dot_utilities_dot_webhook__pb2.WebhookDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.utilities.webhook.WebhookService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class WebhookService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def WebhookCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookCreate",
            v1_dot_utilities_dot_webhook__pb2.WebhookCreateRequest.SerializeToString,
            v1_dot_utilities_dot_webhook__pb2.WebhookCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def WebhookRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookRead",
            v1_dot_utilities_dot_webhook__pb2.WebhookReadRequest.SerializeToString,
            v1_dot_utilities_dot_webhook__pb2.WebhookReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def WebhookUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookUpdate",
            v1_dot_utilities_dot_webhook__pb2.WebhookUpdateRequest.SerializeToString,
            v1_dot_utilities_dot_webhook__pb2.WebhookUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def WebhookDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.webhook.WebhookService/WebhookDelete",
            v1_dot_utilities_dot_webhook__pb2.WebhookDeleteRequest.SerializeToString,
            v1_dot_utilities_dot_webhook__pb2.WebhookDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
